package com.lowagie.test.functional;

import com.lowagie.text.Document;
import com.lowagie.text.DocWriter;
import com.lowagie.text.Rectangle;
import com.lowagie.text.xml.XmlWriter;
import com.lowagie.text.pdf.PdfWriter;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.*;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;

public abstract class FunctionalTestSuite {
    protected String description = "";
    protected Document document = null;
    protected DocWriter writer = null;
    protected XmlWriter debugWriter = null;
    protected Rectangle pageSize = null;


    private String name = this.getClass().getName().substring(this.getClass().getName().lastIndexOf(".")+1);

    public abstract void startTesting(OutputStream os) throws Exception;
    public abstract void startDebugging(OutputStream os) throws Exception;
    public abstract void stopTesting();



    public void test(OutputStream os ) throws Exception {
        startTesting(os);
        Class suiteClass = this.getClass();
        List testNames = getTestNames();
        for (Iterator it = testNames.iterator(); it.hasNext();) {
            String nextName = (String) it.next();
            runTest(nextName);
        }
        stopTesting();
    }

    public void test(String testName, OutputStream os) throws Exception {
        startTesting(os);
        runTest(testName);
        stopTesting();
    }

    public void runTest(String testName) throws Exception {
        Class suiteClass = this.getClass();
        Method test = suiteClass.getMethod(testName, null);
        test.invoke(this, null);
    }

    public void debug(String testName, OutputStream os) throws Exception {
        startDebugging(os);
        Class suiteClass = this.getClass();
        Method test = suiteClass.getMethod(testName, null);
        test.invoke(this, null);
        stopTesting();
    }

    public List getTestNames() {
        ArrayList results = new ArrayList();
        Class suiteClass = this.getClass();
        Method[] methods = suiteClass.getMethods();
        for (int i=0; i<methods.length; i++) {
            Method nextMethod = methods[i];
            if (Modifier.isPublic(nextMethod.getModifiers())) {
                Class[] params = nextMethod.getParameterTypes();
                if (params.length == 0) {
                    String name = nextMethod.getName();
                    if (name.startsWith("test")) {
                        results.add(name);
                    }
                }
            }
        }
        return results;
    }



    public String getDescription() {
        return description;
    }

    public String getName() {
        return name;
    }



    private void testSuite(Object suite) throws Exception {

        Class suiteClass = suite.getClass();
    }

}
