package com.lowagie.test.functional.pdf;

import com.lowagie.text.*;
import com.lowagie.text.Font;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.*;

import java.io.IOException;
import java.io.OutputStream;
import java.awt.*;
import java.util.Random;

public class MultiColumnTextIrregularTestSuite extends PdfBaseTestSuite {
    private String description = "Tests MultiColumnText with Irregular borders";
    Random rand = new Random();

    public MultiColumnTextIrregularTestSuite() {
        pageSize = PageSize.LETTER;
    }

    public MultiColumnTextIrregularTestSuite(OutputStream os) throws DocumentException {
        super(PageSize.LETTER);
    }

    public void testFullPage() throws DocumentException{
        document.newPage();

       // calculate diamond shaped hole
       float diamondHeight = 400;
       float diamondWidth = 400;
       float gutter = 10;
       float bodyHeight = document.top() - document.bottom();
       float colMaxWidth = (document.right() - document.left() - (gutter * 2)) / 2f;
       float diamondTop = document.top() - ((bodyHeight - diamondHeight) / 2f);
       float diamondInset = colMaxWidth - (diamondWidth / 2f);
       float centerX = (document.right() - document.left()) / 2 + document.left();
       // draw stuff
       PdfContentByte cb = writer.getDirectContentUnder();
       cb.setLineWidth(5);
       cb.setColorStroke(Color.GRAY);
       cb.moveTo(centerX , document.top());
       cb.lineTo(centerX, document.bottom());
       cb.stroke();

       cb.moveTo(centerX, diamondTop);
       cb.lineTo(centerX - (diamondWidth/2), diamondTop - (diamondHeight / 2));
       cb.lineTo(centerX, diamondTop - diamondHeight);
       cb.lineTo(centerX + (diamondWidth/2), diamondTop - (diamondHeight / 2));
       cb.lineTo(centerX, diamondTop);
       cb.setColorFill(Color.GRAY);
       cb.fill();

       MultiColumnText mct = new MultiColumnText(document.top() - document.bottom());

       // setup column 1
       float[] left = {document.left(), document.top(), document.left(), document.bottom()};
       float[] right = {document.left() + colMaxWidth, document.top(),
                        document.left() + colMaxWidth, diamondTop,
                        document.left() + diamondInset, diamondTop - diamondHeight / 2,
                        document.left() + colMaxWidth, diamondTop - diamondHeight,
                        document.left() + colMaxWidth, document.bottom()
                       };
       mct.addColumn(left, right);

       // setup column 2
       left = new float[] { document.right() - colMaxWidth, document.top(),
                             document.right() - colMaxWidth, diamondTop,
                             document.right() - diamondInset, diamondTop - diamondHeight / 2,
                             document.right() - colMaxWidth, diamondTop - diamondHeight,
                             document.right() - colMaxWidth, document.bottom()
                            };
       right = new float[] { document.right(), document.top(), document.right(), document.bottom() };
       mct.addColumn(left, right);

       // add text
       for (int i=0; i<8; i++) {
          mct.addElement(newPara(generateText(1), Element.ALIGN_JUSTIFIED_ALL, Font.NORMAL));
       }
       document.add(mct);
    }


    private Element newPara(String text, int alignment, int type) {
       Font font = FontFactory.getFont("Helvetica", 10, type, Color.BLACK);
       Paragraph p = new Paragraph(text, font);
       //Phrase p = new Phrase(text, font);
       p.setAlignment(alignment);
       p.setLeading(font.size() * 1.2f);
       return p;
    }


   String[] fakeLatin =
     {"Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Nullam placerat aliquam leo. Morbi mauris tellus, vulputate nec, feugiat sit amet, eleifend vel, mauris. Nullam augue. Duis tristique. Sed condimentum suscipit risus. Quisque nec orci eu pede tincidunt bibendum. Integer ligula dolor, euismod quis, posuere a, viverra sit amet, augue. Praesent id eros. Mauris id tellus ac sem pharetra dapibus. In hac habitasse platea dictumst. Maecenas aliquam, ante non pulvinar ultricies, felis nunc porta nisl, vel molestie ipsum mauris non tortor. Suspendisse volutpat. Aliquam eros. Praesent fermentum. Maecenas a lectus vel mauris rhoncus lobortis. Curabitur quis risus at magna tempus malesuada. Sed posuere tortor in nulla. Mauris nibh velit, accumsan sed, pellentesque vel, interdum a, lacus. Sed a magna.",
      "Curabitur placerat arcu tempor dui. Nulla ligula enim, sollicitudin a, auctor sed, lacinia et, tellus. Praesent blandit mauris vel nibh. Quisque lacinia, nisl non adipiscing tincidunt, turpis massa pharetra justo, vitae scelerisque sem felis non neque. Aliquam tincidunt tellus ac justo. Nullam pulvinar. Duis condimentum, orci ac dictum semper, ipsum massa varius magna, id gravida elit lacus eget risus. Curabitur rhoncus nulla at enim. Nulla mi velit, semper vitae, ultricies id, posuere iaculis, nisl. Cras ultrices ipsum quis eros. Integer condimentum pulvinar tellus. Fusce lobortis enim at est. Praesent sit amet enim. Vestibulum sit amet magna. Vestibulum neque nunc, auctor eget, porta sit amet, faucibus sed, tortor. Curabitur vel mauris. \n",
      "Fusce a augue a velit consectetuer adipiscing. Donec luctus dui vel mauris. Nullam nec mauris vitae lacus vehicula mollis. In hac habitasse platea dictumst. Nulla metus massa, laoreet in, laoreet quis, eleifend id, enim. Proin convallis. Duis euismod interdum elit. Mauris dignissim est sed tellus. Suspendisse potenti. Sed dictum cursus tellus. Nam sodales. \n",
      "Donec sed diam in arcu iaculis dapibus. In gravida justo at odio. Pellentesque at est. Donec commodo odio nec tellus. Donec mattis elementum augue. Donec varius arcu ut est. Proin rhoncus. Curabitur ac augue a elit sodales egestas. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Maecenas mauris tortor, hendrerit eget, varius id, ornare vitae, libero. Nam tincidunt nonummy turpis. Curabitur venenatis, erat a suscipit bibendum, tellus leo gravida ligula, ac bibendum felis lectus a nulla. \n",
      "Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Morbi interdum elit sed orci. Mauris nec nulla. Maecenas urna. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Aenean et eros. Nam sed lectus. Duis felis. Praesent nunc ipsum, consectetuer ac, commodo quis, blandit aliquam, ligula. Suspendisse convallis, nulla sit amet consectetuer elementum, lorem lectus aliquam diam, vel dignissim tortor mi sed ligula. Sed suscipit tortor vel tellus. Cras sollicitudin sapien eu nulla. Integer sapien eros, volutpat a, pellentesque sed, egestas at, leo. Maecenas erat pede, accumsan a, rhoncus eu, pulvinar non, enim. Curabitur nisl. Curabitur euismod quam nec orci. Sed ornare. Mauris ipsum arcu, malesuada non, cursus aliquam, dapibus rutrum, pede. In scelerisque facilisis nibh. Fusce ac nisl nec eros suscipit sodales. \n" };

   String generateText(int numPara) {
      StringBuffer results = new StringBuffer(500);
      for (int i=0; i<numPara; i++) {
         results.append(fakeLatin[rand.nextInt(fakeLatin.length)]);
      }
      return results.toString();
   }
}

