package com.lowagie.test.functional.pdf;

import com.lowagie.text.*;
import com.lowagie.text.Font;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;

import java.io.IOException;
import java.io.OutputStream;
import java.awt.*;

public class NestedTableTestSuite extends PdfBaseTestSuite {
    private String description = "Tests nested com.lowagie.text.Tables when rendered to PDF";

    public NestedTableTestSuite() {
    }

    public NestedTableTestSuite(OutputStream os) throws DocumentException {
        super(PageSize.LETTER);
    }

    public void testSimpleNesting() throws DocumentException
    {
        document.newPage();
        Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);
        Table outerTable = new Table(1);
        outerTable.setWidth(100);
        outerTable.setBorderWidth(0);
        Paragraph p = new Paragraph("test", tableFont);

        Table innerTable= new Table(1);
        innerTable.setWidth(100);
        innerTable.setBorderWidth(0);
        innerTable.addCell(makeCell(new Element[] {p }, Element.ALIGN_LEFT, Element.ALIGN_TOP));

        outerTable.addCell(makeCell(new Element[] {p, innerTable}, Element.ALIGN_LEFT, Element.ALIGN_TOP));
        document.add(outerTable);
    }

    /*
    public void testMultipleNesting() throws DocumentException  {
        try {
        Table outerTable = new Table(1);
        outerTable.setWidth(100f);

        // If the widths on these two tables aren't the same, it will
        //   cause an ArrayIndexOutOfBoundsException at Table.mergeInsertedTables(Table.java:1437)
        //   as of iText-paulo-138
        Table innerTable1= new Table(1);
        innerTable1.setWidth(90f);

        Table innerTable2= new Table(1);
        innerTable2.setWidth(90.1f);


        outerTable.addCell(makeCell(new Element[] {innerTable1}, Element.ALIGN_LEFT, Element.ALIGN_TOP));
        outerTable.addCell(makeCell(new Element[] {innerTable2 }, Element.ALIGN_LEFT, Element.ALIGN_TOP));

        document.add(outerTable);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
*/


    Cell makeCell(Element[] elements, int hAlign, int vAlign)  {
        Cell cell = new Cell();
        cell.setHorizontalAlignment(hAlign);
        cell.setVerticalAlignment(vAlign);
        if (elements != null) {
            for (int i = 0; i < elements.length; i++) {
                Element el = elements[i];
                cell.add(el);
            }
        }
        return cell;
    }

}
