package com.lowagie.test.functional.pdf;

import com.lowagie.text.*;
import com.lowagie.text.Font;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.*;
//import com.controlj.green.web.reports.enhancedreporting.pdf.PdfPTableBorder;

import java.io.IOException;
import java.io.OutputStream;
import java.awt.*;
import java.util.Random;

public class NormalPdfPTableBorderTestSuite extends PdfBaseTestSuite {
    private String description = "Tests normal (non-variable) style borders on com.lowagie.text.Table when rendered to PDF";
    Random rand = new Random();

    public NormalPdfPTableBorderTestSuite() {
        pageSize = PageSize.LETTER;
    }

    public NormalPdfPTableBorderTestSuite(OutputStream os) throws DocumentException {
        super(PageSize.LETTER);
    }

    public void testBorderNormalThin() throws BadElementException, DocumentException, IOException {
        float padding = 0f;
        float borderWidth = 0.5f;
        Rectangle boxBorder = new Rectangle(0f, 0f);
        boxBorder.setBorder(Rectangle.BOX);
        boxBorder.setBorderWidth(borderWidth);
        testBorder(writer, document, padding, boxBorder, 0f);
    }

    public void testBorderNormalThick()  throws BadElementException, DocumentException, IOException
    {
       float padding = 0f;
       float borderWidth = 5f;
       Rectangle boxBorder = new Rectangle(0f, 0f);
       boxBorder.setBorder(Rectangle.BOX);
       boxBorder.setBorderWidth(borderWidth);
       testBorder(writer, document, padding, boxBorder, 0f);
    }

    public void testBorderNormalPadding()  throws BadElementException, DocumentException, IOException
    {
       float padding = 4f;
       float borderWidth = 2f;
       Rectangle boxBorder = new Rectangle(0f, 0f);
       boxBorder.setBorder(Rectangle.BOX);
       boxBorder.setBorderWidth(borderWidth);
       testBorder(writer, document, padding, boxBorder, 0f);
    }

   public void testBorderNormalPaddingSpacing()  throws BadElementException, DocumentException, IOException
   {
      float padding = 0f;
      float borderWidth = 2f;
      Rectangle boxBorder = new Rectangle(0f, 0f);
      boxBorder.setBorder(Rectangle.BOX);
      boxBorder.setBorderWidth(borderWidth);
      testBorder(writer, document, padding, boxBorder, 5f);
   }

   /*
   public void testBorderMultiPage()  throws BadElementException, DocumentException, IOException
   {
      Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);
      int repetitions=40;
      float padding = 0f;
      float borderWidth = 5f;
      Rectangle borders = new Rectangle(0f, 0f);
      borders.setBorder(Rectangle.BOX);
      borders.setBorderWidthLeft(borderWidth);
      borders.setBorderWidthRight(borderWidth);
      borders.setBorderWidthTop(borderWidth);
      borders.setBorderWidthBottom(borderWidth);

      document.newPage();
      PdfPTable table = null;
      table = new PdfPTable(4);
      table.setWidthPercentage(100f);

      PdfPTableBorder tb = new PdfPTableBorder();
      tb.setBorderColor(Color.BLUE);
      tb.setBorderWidth(3f);
      tb.setBorder(Rectangle.BOX);
      table.setTableEvent(tb);

      float rulerTop = document.getPageSize().top();
      addRuler(1f, document.leftMargin() - 10, rulerTop, rulerTop);

      table.addCell(makeCell("1-Top", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
      table.addCell(makeCell("2-Middle", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
      table.addCell(makeCell("3-Bottom", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
      table.addCell(makeCell("4-Has a y", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));

      for (int i=0; i<repetitions; i++) {
         table.addCell(makeCell("5-Abcdy", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
         table.addCell(makeCell("6-Abcdy", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
         table.addCell(makeCell("7-Abcdy", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
         table.addCell(makeCell("8-This\nis\na little\ntaller", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
      }
      document.add(table);
   }
*/

   public void testNestedTableColspan() throws BadElementException, DocumentException, IOException {
       document.newPage();
       Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);
       Rectangle borders = new Rectangle(0f, 0f);
       borders.setBorder(Rectangle.BOX);
       borders.setBorderWidth(0.5f);
       float padding = 0f;


       PdfPTable table = null;
       table = new PdfPTable(4);
       table.setWidthPercentage(100f);
//        table.setBorderWidth(0);
//        table.setBorderColor(new Color(0, 0, 255));
//        table.setPadding(padding);
//        table.setSpacing(spacing);

       table.addCell(makeCell("1-Top", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));

       PdfPTable innerTable = new PdfPTable(2);
       innerTable.setWidthPercentage(100);
       innerTable.addCell(makeCell("TL", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
       innerTable.addCell(makeCell("TR", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
       innerTable.addCell(makeCell("BL", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
       innerTable.addCell(makeCell("BR", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
       PdfPCell spanCell = new PdfPCell(innerTable);
       spanCell.setColspan(2);
       table.addCell(spanCell);
       //table.addCell(makeCell("3-Bottom", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));

       table.addCell(makeCell("4-Has a y", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));

       table.addCell(makeCell("5-Abcdy", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
       table.addCell(makeCell("6-Abcdy", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
       table.addCell(makeCell("7-Abcdy", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
       table.addCell(makeCell("8-This\nis\na little\ntaller", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));

       document.add(table);

       float rulerTop = document.getPageSize().top();
       addRuler(1f, document.leftMargin() - 10, rulerTop, rulerTop);
   }


    public void testSkipFirstHeader() throws DocumentException {
        document.newPage();
        Font font = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);

        PdfPTable table = new PdfPTable(2);
        table.setWidthPercentage(50f);
        table.setHorizontalAlignment(Element.ALIGN_LEFT);
        table.setSplitRows(true);
        //table.setSplitLate(false);

        PdfPCell h1 = new PdfPCell(new Paragraph("Header 1", font));
        PdfPCell h2 = new PdfPCell(new Paragraph("Header 2", font));
        table.setHeaderRows(1);
        table.addCell(h1);
        table.addCell(h2);

        for (int row=1; row <= 1000; row++)
        {
           /*
            if (row % 50 == 49 )
            {
                document.add(table);
                table.deleteBodyRows();
                table.setSkipFirstHeader(true);
            }
            */
            makeNewRow(table, row, font);
        }
        document.add(table);
    }

    public void testMultipleElements() throws DocumentException
    {
       document.newPage();
       Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);
       PdfPTable table = new PdfPTable(1);
       table.setWidthPercentage(100f);


       Paragraph p1 = new Paragraph("This is first", tableFont);
       Paragraph p2 = new Paragraph("This is second", tableFont);
       PdfPCell cell = new PdfPCell(p1);
       //cell.addElement(p1);
       cell.addElement(p2);
       table.addCell(cell);
       document.add(table);

    }

    public void testNestedBig() throws DocumentException
    {
       document.newPage();
       Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);
       PdfPTable table = new PdfPTable(1);
       table.setSplitRows(true);
       table.setWidthPercentage(100f);

       PdfPTable innerTable = new PdfPTable(1);
       for (int i=0; i<20; i++)
       {
          Paragraph p = new Paragraph(generateText(1), tableFont);
          PdfPCell innerCell = new PdfPCell(p);
          innerTable.addCell(innerCell);
       }
       table.addCell(new PdfPCell(innerTable));
/*
       Paragraph p1 = new Paragraph("This is first", tableFont);
       PdfPCell cell1 = new PdfPCell(p1);

       Paragraph p2 = new Paragraph("This is second", tableFont);
       PdfPCell cell2 = new PdfPCell(p2);

       PdfPTable table2 = new PdfPTable(1);
       PdfPTable table3 = new PdfPTable(1);

       table2.addCell(cell1);
       table3.addCell(cell2);

       PdfPCell containerCell = new PdfPCell();
       containerCell.addElement(table2);
       containerCell.addElement(table3);
       table.addCell(containerCell);
  */
       document.add(table);

    }

    private void makeNewRow(PdfPTable table, int row, Font font) {
        PdfPCell cell;

        cell = new PdfPCell(new Paragraph(String.valueOf(row), font));
        table.addCell(cell);
        cell = new PdfPCell(new Paragraph("Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Nulla mauris nibh, ultricies nec, adipiscing eget.", font));
        table.addCell(cell);
    }

    private void testBorder(PdfWriter writer, Document document, float padding, Rectangle border, float spacing) throws BadElementException, DocumentException, IOException {
        Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);
        BaseFont baseFont = tableFont.getBaseFont();
        float targetOffset = 0.2f / 2f;  // 1/2 of line width for target
        float leading = tableFont.size() * 1.2f;

        makeTestPage(tableFont, border, writer, document, spacing, padding);
        addText("\n" + (border.isUseVariableBorders() ? "Variable" : "Normal") + " border, cellspacing=" + spacing + ", cellpadding=" + padding, Element.ALIGN_CENTER);
        float tableTop = document.getPageSize().height() - document.topMargin();
        float tableLeft = document.leftMargin();

        //outside top left corner
        float markerLeft = tableLeft - targetOffset + spacing -
                border.borderWidth()/2f;
        float markerTop = tableTop + targetOffset - spacing +
                border.borderWidth()/2f;;

        float markerX = markerLeft;
        float markerY = markerTop;

        drawTarget(markerX, markerY, 10);

        // outside top right corner
        float markerRight = document.getPageSize().width() - document.rightMargin() + targetOffset - spacing +
                border.borderWidth()/2f;
        drawTarget(markerRight, markerTop, 10);

        // inside upper left corner of cell 5
        float tableRow1Height = baseFont.getFontDescriptor(BaseFont.ASCENT, 8) -
                baseFont.getFontDescriptor(BaseFont.DESCENT, 8) +
                (padding * 2f) + (2f * spacing) +
                2f * border.borderWidth();

        markerX = markerLeft + targetOffset + border.borderWidth();
        markerY -= tableRow1Height + targetOffset * 2f;
        drawTarget(markerX, markerY, 10);

        // outside bottom left corner
        float markerBottom = markerY - 
                baseFont.getFontDescriptor(BaseFont.ASCENT, 8)+
                baseFont.getFontDescriptor(BaseFont.DESCENT, 8) -
                (leading * 3f +
                border.borderWidth() +
                2f * padding);
        drawTarget(markerLeft, markerBottom, 10);

        // outside bottom right corner
        drawTarget(markerRight, markerBottom, 10);

        addText("Checkpoints:", Element.ALIGN_LEFT);
        addText("1) Outside all four corners", Element.ALIGN_LEFT);
        addText("2) Inside the upper left corner of cell 5", Element.ALIGN_LEFT);
    }


    private void makeTestPage(Font tableFont, Rectangle borders, PdfWriter writer, Document document,
                              float spacing, float padding) throws BadElementException, DocumentException, IOException {
        document.newPage();
        PdfPTable table = null;
        table = new PdfPTable(4);
        table.setWidthPercentage(100f);
//        table.setBorderWidth(0);
//        table.setBorderColor(new Color(0, 0, 255));
//        table.setPadding(padding);
//        table.setSpacing(spacing);
       

        table.addCell(makeCell("1-Top", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
        table.addCell(makeCell("2-Middle", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
        table.addCell(makeCell("3-Bottom", Element.ALIGN_BOTTOM, Element.ALIGN_CENTER, tableFont, tableFont.size() * 1.2f, padding, borders));
        table.addCell(makeCell("4-Has a y", Element.ALIGN_TOP, Element.ALIGN_RIGHT, tableFont, tableFont.size() * 1.2f, padding, borders));

        table.addCell(makeCell("5-Abcdy", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
        table.addCell(makeCell("6-Abcdy", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
        table.addCell(makeCell("7-Abcdy", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
        table.addCell(makeCell("8-This\nis\na little\ntaller", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
/*
        table.addCell(makeCell("X", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
        table.addCell(makeCell("X", Element.ALIGN_TOP, Element.ALIGN_CENTER, tableFont, tableFont.size() * 1.2f, padding, borders));
        table.addCell(makeCell("X", Element.ALIGN_TOP, Element.ALIGN_RIGHT, tableFont, tableFont.size() * 1.2f, padding, borders));
        table.addCell(makeCell("X", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, padding, borders));
*/

        document.add(table);

        float rulerTop = document.getPageSize().top();
        addRuler(1f, document.leftMargin() - 10, rulerTop, rulerTop);
    }



    PdfPCell makeCell(String text, int vAlignment, int hAlignment, Font font, float leading, float padding, Rectangle borders) {
        Paragraph p = new Paragraph(text, font);
        p.setAlignment(hAlignment);
        //p.setLeading(leading);

        PdfPCell cell = new PdfPCell(p);
        cell.setPadding(padding);
        cell.setLeading(leading, 0);
        cell.setVerticalAlignment(vAlignment);
        cell.setHorizontalAlignment(hAlignment);
        cell.cloneNonPositionParameters(borders);
        cell.setUseDescender(true);
        cell.setUseAscender(true);
        cell.setUseBorderPadding(true);
        return cell;
    }

   String[] fakeLatin =
     {"Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Nullam placerat aliquam leo. Morbi mauris tellus, vulputate nec, feugiat sit amet, eleifend vel, mauris. Nullam augue. Duis tristique. Sed condimentum suscipit risus. Quisque nec orci eu pede tincidunt bibendum. Integer ligula dolor, euismod quis, posuere a, viverra sit amet, augue. Praesent id eros. Mauris id tellus ac sem pharetra dapibus. In hac habitasse platea dictumst. Maecenas aliquam, ante non pulvinar ultricies, felis nunc porta nisl, vel molestie ipsum mauris non tortor. Suspendisse volutpat. Aliquam eros. Praesent fermentum. Maecenas a lectus vel mauris rhoncus lobortis. Curabitur quis risus at magna tempus malesuada. Sed posuere tortor in nulla. Mauris nibh velit, accumsan sed, pellentesque vel, interdum a, lacus. Sed a magna.",
      "Curabitur placerat arcu tempor dui. Nulla ligula enim, sollicitudin a, auctor sed, lacinia et, tellus. Praesent blandit mauris vel nibh. Quisque lacinia, nisl non adipiscing tincidunt, turpis massa pharetra justo, vitae scelerisque sem felis non neque. Aliquam tincidunt tellus ac justo. Nullam pulvinar. Duis condimentum, orci ac dictum semper, ipsum massa varius magna, id gravida elit lacus eget risus. Curabitur rhoncus nulla at enim. Nulla mi velit, semper vitae, ultricies id, posuere iaculis, nisl. Cras ultrices ipsum quis eros. Integer condimentum pulvinar tellus. Fusce lobortis enim at est. Praesent sit amet enim. Vestibulum sit amet magna. Vestibulum neque nunc, auctor eget, porta sit amet, faucibus sed, tortor. Curabitur vel mauris. \n",
      "Fusce a augue a velit consectetuer adipiscing. Donec luctus dui vel mauris. Nullam nec mauris vitae lacus vehicula mollis. In hac habitasse platea dictumst. Nulla metus massa, laoreet in, laoreet quis, eleifend id, enim. Proin convallis. Duis euismod interdum elit. Mauris dignissim est sed tellus. Suspendisse potenti. Sed dictum cursus tellus. Nam sodales. \n",
      "Donec sed diam in arcu iaculis dapibus. In gravida justo at odio. Pellentesque at est. Donec commodo odio nec tellus. Donec mattis elementum augue. Donec varius arcu ut est. Proin rhoncus. Curabitur ac augue a elit sodales egestas. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Maecenas mauris tortor, hendrerit eget, varius id, ornare vitae, libero. Nam tincidunt nonummy turpis. Curabitur venenatis, erat a suscipit bibendum, tellus leo gravida ligula, ac bibendum felis lectus a nulla. \n",
      "Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Morbi interdum elit sed orci. Mauris nec nulla. Maecenas urna. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Aenean et eros. Nam sed lectus. Duis felis. Praesent nunc ipsum, consectetuer ac, commodo quis, blandit aliquam, ligula. Suspendisse convallis, nulla sit amet consectetuer elementum, lorem lectus aliquam diam, vel dignissim tortor mi sed ligula. Sed suscipit tortor vel tellus. Cras sollicitudin sapien eu nulla. Integer sapien eros, volutpat a, pellentesque sed, egestas at, leo. Maecenas erat pede, accumsan a, rhoncus eu, pulvinar non, enim. Curabitur nisl. Curabitur euismod quam nec orci. Sed ornare. Mauris ipsum arcu, malesuada non, cursus aliquam, dapibus rutrum, pede. In scelerisque facilisis nibh. Fusce ac nisl nec eros suscipit sodales. \n" };

   String generateText(int numPara) {
      StringBuffer results = new StringBuffer(500);
      for (int i=0; i<numPara; i++) {
         results.append(fakeLatin[rand.nextInt(fakeLatin.length)]);
      }
      return results.toString();
   }


}
