package com.lowagie.test.functional.pdf;

import com.lowagie.text.*;
import com.lowagie.text.Font;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;

import java.io.IOException;
import java.io.OutputStream;
import java.awt.*;

public class NormalTableBorderTestSuite extends PdfBaseTestSuite {
    private String description = "Tests normal (non-variable) style borders on com.lowagie.text.Table when rendered to PDF";

    public NormalTableBorderTestSuite() {
        pageSize = PageSize.LETTER;
    }

    public NormalTableBorderTestSuite(OutputStream os) throws DocumentException {
        super(PageSize.LETTER);
    }

    public void testBorderNormalThin() throws BadElementException, DocumentException, IOException {
        float padding = 0f;
        float spacing = 0f;
        float borderWidth = 0.5f;
        Rectangle boxBorder = new Rectangle(0f, 0f);
        boxBorder.setBorder(Rectangle.BOX);
        boxBorder.setBorderWidth(borderWidth);
        testBorder(writer, document, padding, spacing, boxBorder, 1);

        //testDirectWrite();
    }

    private void testDirectWrite()
    {
       PdfContentByte cb = writer.getDirectContentUnder();
       cb.setLineWidth(5);
       cb.moveTo(400,400);
       cb.setColorStroke(Color.RED);
       cb.lineTo(500,400);
       cb.setColorStroke(Color.BLUE);
       cb.lineTo(500,500);
       cb.stroke();
    }

    public void testBorderNormalThick()  throws BadElementException, DocumentException, IOException
    {
       float padding = 0f;
       float spacing = 0f;
       float borderWidth = 5f;
       Rectangle boxBorder = new Rectangle(0f, 0f);
       boxBorder.setBorder(Rectangle.BOX);
       boxBorder.setBorderWidth(borderWidth);
       testBorder(writer, document, padding, spacing, boxBorder, 1);
    }

    public void testBorderNormalSpacing()  throws BadElementException, DocumentException, IOException
    {
       float padding = 0f;
       float spacing = 5f;
       float borderWidth = 2f;
       Rectangle boxBorder = new Rectangle(0f, 0f);
       boxBorder.setBorder(Rectangle.BOX);
       boxBorder.setBorderWidth(borderWidth);
       testBorder(writer, document, padding, spacing, boxBorder, 8);
    }

   public void testBorderSpacingMultiPage()  throws BadElementException, DocumentException, IOException
    {
       float padding = 0f;
       float spacing = 0f;
       float borderWidth = 1f;
       Rectangle boxBorder = new Rectangle(0f, 0f);
       boxBorder.setBorder(Rectangle.BOX);
       boxBorder.setBorderWidth(borderWidth);
       Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);

       testBorder(writer, document, padding, spacing, boxBorder, 20);
    }

    /*
   public void testEmptyCell() throws DocumentException
   {
      Table table = null;
      table = new Table(2);
      table.setDefaultCellBorder(Rectangle.TOP | Rectangle.BOTTOM | Rectangle.LEFT | Rectangle.RIGHT);

	   table.setDefaultCellBorderWidth(1);
		table.setDefaultCellBorderColor(new Color(0,0,0));
      table.setWidth(100);

      Cell cell;
      cell = new Cell(new Phrase("This is a test"));
      table.addCell(cell);

      cell = new Cell();
      table.addCell(cell);


      cell = new Cell(new Phrase("This is a test"));
      table.addCell(cell);

      cell = new Cell();
      table.addCell(cell);

      document.add(table);
   }
   */

    public void testCellBackgrounds() throws DocumentException    {
        document.newPage();
        Table table = new Table(2, 2);
        table.setBorder(Rectangle.NO_BORDER);
        Cell c = new Cell(new Phrase("Box Border 1"));
        c.setBorder(Rectangle.BOX);
        c.setBorderWidth(.2f);
        table.addCell(c);
        c = new Cell(new Phrase("Box Border 2"));
        c.setBorder(Rectangle.BOX);
        c.setBorderWidth(.2f);
        table.addCell(c);
        c = new Cell(new Phrase("Yellow BG, No Border"));
        c.setBorder(Rectangle.NO_BORDER);
        c.setBackgroundColor(new Color(255, 255, 0));
        table.addCell(c);
        c = new Cell(new Phrase("No BG, No Border"));
        c.setBorder(Rectangle.NO_BORDER);
        table.addCell(c);

        document.add(table);

        addText("\nCheckpoints:", Element.ALIGN_LEFT);
        addText("1) Complete box border around top two cells", Element.ALIGN_LEFT);
        addText("2) Yellow background on bottom left cell", Element.ALIGN_LEFT);
    }


    private void testBorder(PdfWriter writer, Document document, float padding, float spacing, Rectangle border, int rows) throws BadElementException, DocumentException, IOException {
        Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);
        BaseFont baseFont = tableFont.getBaseFont();
        float targetOffset = 0.2f / 2f;  // 1/2 of line width for target

        makeTestPage(tableFont, border, writer, document, spacing, padding, rows);
        addText("\n" + (border.isUseVariableBorders() ? "Variable" : "Normal") + " border, cellspacing=" + spacing + ", cellpadding=" + padding, Element.ALIGN_CENTER);
        float tableTop = document.getPageSize().height() - document.topMargin();
        float tableLeft = document.leftMargin();

        //outside top left corner
        float markerLeft = tableLeft - targetOffset + spacing -
                border.borderWidth()/2f;
        float markerTop = tableTop + targetOffset - spacing +
                border.borderWidth()/2f;;

        float markerX = markerLeft;
        float markerY = markerTop;

        drawTarget(markerX, markerY, 10);

        // outside top right corner
        float markerRight = document.getPageSize().width() - document.rightMargin() + targetOffset - spacing +
                border.borderWidth()/2f;;
        drawTarget(markerRight, markerTop, 10);

        // inside upper left corner of cell 5
        float tableRow1Height = baseFont.getFontDescriptor(BaseFont.ASCENT, 8) -
                baseFont.getFontDescriptor(BaseFont.DESCENT, 8) +
                (padding * 2f) + (2f * spacing) +
                2f * border.borderWidth();

        markerX = markerLeft + targetOffset + border.borderWidth();
        markerY -= tableRow1Height + targetOffset * 2f;
        drawTarget(markerX, markerY, 10);

        // outside bottom left corner
        float markerBottom = markerY -
                (baseFont.getFontDescriptor(BaseFont.ASCENT, 8) +
                (tableFont.size() * 1.2f * 3) -
                baseFont.getFontDescriptor(BaseFont.DESCENT, 8) +
                border.borderWidth() +
                2f * padding);
        drawTarget(markerLeft, markerBottom, 10);

        // outside bottom right corner
        drawTarget(markerRight, markerBottom, 10);

        addText("Checkpoints:", Element.ALIGN_LEFT);
        addText("1) Outside all four corners", Element.ALIGN_LEFT);
        addText("2) Inside the upper left corner of cell 5", Element.ALIGN_LEFT);
    }

    protected Table makeTestTable(float padding, float spacing) throws BadElementException {
        Table table = null;
        table = new Table(4);
        table.setCellsFitPage(true);
        table.setWidth(100f);
        table.setBorderWidth(1);
        table.setBorderColor(new Color(255, 0, 0));
        table.setPadding(padding);
        table.setSpacing(spacing);
        return table;
    }

    private void makeTestPage(Font tableFont, Rectangle borders, PdfWriter writer, Document document,
                              float spacing, float padding, int rows) throws BadElementException, DocumentException, IOException {
        document.newPage();

       float rulerTop = document.getPageSize().top();
       addRuler(1f, document.leftMargin() - 10, rulerTop, rulerTop);

        Table table = makeTestTable(padding, spacing);


        Cell cell = makeCell("1-Top", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders);
        cell.setHeader(true);
        table.addCell(cell);
        cell = makeCell("2-Middle", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders);
        cell.setHeader(true);
        table.addCell(cell);
        cell = makeCell("3-Bottom", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders);
        cell.setHeader(true);
        table.addCell(cell);
        cell = makeCell("4-Has a y", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders);
        cell.setHeader(true);
        table.addCell(cell);
        table.endHeaders();

       for (int i=0; i<rows; i++)
       {
        table.addCell(makeCell("5-Abcdy", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        //table.addCell(makeCell("6-Abcdy", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell(null, Element.ALIGN_TOP, Element.ALIGN_CENTER, borders));
        table.addCell(makeCell("7-Abcdy", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("8-This\nis\na little\ntaller", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
       }
        /*
        table.addCell(makeCell("X", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("X", Element.ALIGN_TOP, Element.ALIGN_CENTER, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("X", Element.ALIGN_TOP, Element.ALIGN_RIGHT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("X", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        */
        document.add(table);

        addRuler(1f, document.leftMargin() - 10, rulerTop, rulerTop);
    }

    Cell makeCell(String text, int vAlignment, int hAlignment, Font font, float leading, Rectangle borders) {
        Paragraph p = new Paragraph(text, font);
        p.setLeading(leading);
        p.setAlignment(hAlignment);
        return makeCell(new Element[]{p}, vAlignment, hAlignment, borders);
    }

    Cell makeCell(Element[] elements, int vAlignment, int hAlignment, Rectangle borders) {
        Cell cell = new Cell();

        if (elements != null) {
            for (int i = 0; i < elements.length; i++) {
                Element el = elements[i];
                cell.add(el);
            }
        }
        cell.setVerticalAlignment(vAlignment);
        cell.setHorizontalAlignment(hAlignment);
        cell.cloneNonPositionParameters(borders);
        cell.setUseAscender(true);
        cell.setUseDescender(true);
        cell.setUseBorderPadding(true);
        return cell;
    }

}
