package com.lowagie.test.functional.pdf;

import com.lowagie.text.*;
import com.lowagie.text.Font;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;

import java.io.IOException;
import java.io.OutputStream;
import java.awt.*;

public class VariableTableBorderTestSuite extends PdfBaseTestSuite {
    private String description = "Tests Variable style borders on com.lowagie.text.Table when rendered to PDF";

    public VariableTableBorderTestSuite() {
        pageSize = PageSize.LETTER;
    }

    public VariableTableBorderTestSuite(OutputStream os) throws DocumentException {
        super(PageSize.LETTER);
    }


    public void testBorderRainbow() throws BadElementException, DocumentException, IOException {
        float padding = 0f;
        float spacing = 0f;
        Rectangle border = new Rectangle(0f, 0f);
        border.setBorderWidthLeft(6f);
        border.setBorderWidthBottom(5f);
        border.setBorderWidthRight(4f);
        border.setBorderWidthTop(2f);
        border.setBorderColorLeft(Color.RED);
        border.setBorderColorBottom(Color.ORANGE);
        border.setBorderColorRight(Color.YELLOW);
        border.setBorderColorTop(Color.GREEN);
        testBorder(padding, spacing, 1, border);
    }

    public void testBorderPadding() throws BadElementException, DocumentException, IOException {
        float padding = 1f;
        float spacing = 0f;
        Rectangle border = new Rectangle(0f, 0f);
        border.setBorderWidthLeft(.1f);
        border.setBorderWidthBottom(1f);
        border.setBorderWidthRight(.1f);
        border.setBorderWidthTop(1f);
        border.setBorderColorLeft(Color.BLACK);
        border.setBorderColorBottom(Color.LIGHT_GRAY);
        border.setBorderColorRight(Color.BLACK);
        border.setBorderColorTop(Color.LIGHT_GRAY);
        testBorder(padding, spacing, 1, border);
    }

    public void testBorderPaddingSpacing() throws BadElementException, DocumentException, IOException {
        float padding = 2f;
        float spacing = 1f;
        Rectangle border = new Rectangle(0f, 0f);
        border.setBorderWidthLeft(.5f);
        border.setBorderWidthBottom(.5f);
        border.setBorderWidthRight(.5f);
        border.setBorderWidthTop(.5f);
        border.setBorderColorLeft(Color.BLACK);
        border.setBorderColorBottom(Color.BLACK);
        border.setBorderColorRight(Color.BLACK);
        border.setBorderColorTop(Color.BLACK);
        testBorder(padding, spacing, 1, border);
    }


    public void testBorderSpacing() throws BadElementException, DocumentException, IOException {
        float padding = 0f;
        float spacing = 2f;
        Rectangle border = new Rectangle(0f, 0f);
        border.setBorderWidthLeft(.5f);
        border.setBorderWidthBottom(1f);
        border.setBorderWidthRight(.5f);
        border.setBorderWidthTop(1f);
        border.setBorderColorLeft(Color.BLACK);
        border.setBorderColorBottom(Color.LIGHT_GRAY);
        border.setBorderColorRight(Color.BLACK);
        border.setBorderColorTop(Color.LIGHT_GRAY);
        testBorder(padding, spacing, 20, border);
    }

    public void testBorderSquareEnds() throws BadElementException, DocumentException, IOException {
        float padding = 0f;
        float spacing = 0f;
        Rectangle border = new Rectangle(0f, 0f);
        border.setBorderWidthBottom(5f);
        border.setBorderWidthLeft(0f);
        border.setBorderWidthTop(0f);
        border.setBorderWidthRight(0f);
        //border.setBorderColorLeft(Color.BLACK);
        border.setBorderColorBottom(Color.LIGHT_GRAY);
        border.setBorderColorRight(Color.BLACK);
        //border.setBorderColorTop(Color.LIGHT_GRAY);
        testBorder(padding, spacing, 1, border);
    }

    private void testBorder(float padding, float spacing, int rows, Rectangle border) throws BadElementException, DocumentException, IOException {
        Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);
        BaseFont baseFont = tableFont.getBaseFont();
        PdfContentByte cb = writer.getDirectContentUnder();
        float targetOffset = 0.2f / 2f;  // 1/2 of line width for target

        makeTestPage(tableFont, border, writer, document, spacing, padding, rows);
        addText("\n" + (border.isUseVariableBorders() ? "Variable" : "Normal") + " border, cellspacing=" + spacing + ", cellpadding=" + padding, Element.ALIGN_CENTER);
        float tableTop = document.getPageSize().height() - document.topMargin();
        float tableLeft = document.leftMargin();

        //outside top left corner
        float markerLeft = tableLeft - targetOffset + spacing -
                (border.isUseVariableBorders() ? 0f : border.getBorderWidthLeft() / 2f);
        float markerTop = tableTop + targetOffset - spacing +
                (border.isUseVariableBorders() ? 0f : border.getBorderWidthTop() / 2f);

        float markerX = markerLeft;
        float markerY = markerTop;

        drawTarget(markerX, markerY, 10);

        // outside top right corner
        float markerRight = document.getPageSize().width() - document.rightMargin() + targetOffset - spacing +
                (border.isUseVariableBorders() ? 0f : border.getBorderWidthRight() / 2f);
        drawTarget(markerRight, markerTop, 10);

        // inside upper left corner of cell 5
        float tableRow1Height = baseFont.getFontDescriptor(BaseFont.ASCENT, 8) -
                baseFont.getFontDescriptor(BaseFont.DESCENT, 8) +
                (padding * 2f) + (2f * spacing) +
                border.getBorderWidthTop() + border.getBorderWidthBottom() +
                (border.isUseVariableBorders() ? border.getBorderWidthTop() : 0f);

        markerX = markerLeft + targetOffset + border.getBorderWidthLeft();
        markerY -= tableRow1Height + targetOffset * 2f;
        drawTarget(markerX, markerY, 10);

        // outside bottom left corner
        float markerBottom = markerY -
                (baseFont.getFontDescriptor(BaseFont.ASCENT, 8) +
                (tableFont.size() * 1.2f * 3) -
                baseFont.getFontDescriptor(BaseFont.DESCENT, 8) +
                border.getBorderWidthBottom() +
                2f * padding);
        drawTarget(markerLeft, markerBottom, 10);

        // outside bottom right corner
        drawTarget(markerRight, markerBottom, 10);

        addText("Checkpoints:", Element.ALIGN_LEFT);
        addText("1) Outside all four corners", Element.ALIGN_LEFT);
        addText("2) Inside the upper left corner of cell 5", Element.ALIGN_LEFT);
    }

    protected Table makeTestTable(float padding, float spacing) throws BadElementException {
        Table table = null;
        table = new Table(4);
        table.setWidth(100f);
        table.setBorderWidth(0);
        table.setBorderColor(new Color(0, 0, 255));
        table.setPadding(padding);
        table.setSpacing(spacing);
        return table;
    }

    private void makeTestPage(Font tableFont, Rectangle borders, PdfWriter writer, Document document,
                              float spacing, float padding, int rows) throws BadElementException, DocumentException, IOException {
       document.newPage();
       Table table = makeTestTable(padding, spacing);

       Cell cell = makeCell("1-Top", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders);
       cell.setHeader(true);
       table.addCell(cell);
       cell = makeCell("2-Middle", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders);
       cell.setHeader(true);
       table.addCell(cell);
       cell = makeCell("3-Bottom", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders);
       cell.setHeader(true);
       table.addCell(cell);
       cell = makeCell("4-Has a y", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders);
       cell.setHeader(true);
       table.addCell(cell);
       table.endHeaders();

      for (int i=0; i<rows; i++)
      {
       table.addCell(makeCell("5-Abcdy", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
       table.addCell(makeCell("6-Abcdy", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
       table.addCell(makeCell("7-Abcdy", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
       table.addCell(makeCell("8-This\nis\na little\ntaller", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
      }
       /*
        table.addCell(makeCell("1-Top", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("2-Middle", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("3-Bottom", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("4-Has a y", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));

        table.addCell(makeCell("5-Abcdy", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("6-Abcdy", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("7-Abcdy", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        table.addCell(makeCell("8-This\nis\na little\ntaller", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, tableFont.size() * 1.2f, borders));
        */
        document.add(table);

        float rulerTop = document.getPageSize().top();
        addRuler(1f, document.leftMargin() - 10, rulerTop, rulerTop);
    }

    Cell makeCell(String text, int vAlignment, int hAlignment, Font font, float leading, Rectangle borders) {
        Paragraph p = new Paragraph(text, font);
        p.setLeading(leading);
        return makeCell(new Element[]{p}, vAlignment, hAlignment, borders);
    }

    Cell makeCell(Element[] elements, int vAlignment, int hAlignment, Rectangle borders) {
        Cell cell = new Cell();
        if (elements != null) {
            for (int i = 0; i < elements.length; i++) {
                Element el = elements[i];
                cell.add(el);
            }
        }
        cell.setVerticalAlignment(vAlignment);
        cell.setHorizontalAlignment(hAlignment);
        cell.cloneNonPositionParameters(borders);
        cell.setUseAscender(true);
        cell.setUseDescender(true);
        cell.setUseBorderPadding(true);
        return cell;
    }

}
